import math
import psyco
psyco.full()

#plugin information
"""
info = {}
info['name'] = "Vecmath EventScripts python library"
info['version'] = "2.0b"
info['author'] = "GODJonez"
info['url'] = "http://www.eventscripts.com/pages/Vecmath/"
info['description'] = "Provides class and functions for handling vectors for Source"
info['tags'] = ""
"""

class vector:
    def __init__(self, vl):
        # initialize
        # parameters:
        #  vl = list of floating point values or vector string
        try:
            self.vl = [float(x) for x in vl]
        except ValueError:
            self.vl = [float(x) for x in vl.split(",")]
        except TypeError:
            raise TypeError("Cannot convert integer to vector")
        while len(self.vl) < 3:
            self.vl.append(0.0)
    def __repr__(self):
        return "vector(%s)"%str(self.vl)
    def __list__(self):
        # return a list with the vector components
        return list(self.vl)
    def getdict(self):
        # create a dict with the vector components
        return {'x':self.vl[0],'y':self.vl[1],'z':self.vl[2]}
    def __str__(self):
        # str() function
        return self.getstr()
    def getstr(self,sep=","):
        # return a str with components separated with sep
        return str(self.vl[0])+str(sep)+str(self.vl[1])+str(sep)+str(self.vl[2])
    def __len__(self):
        # return the number of components
        return len(self.vl)
    def __add__(self, vl2):
        # vec1 + vec2
        return vector([self[0]+vl2[0], self[1]+vl2[1], self[2]+vl2[2]])
    def __sub__(self, vl2):
        # vec1 - vec2
        return vector([self[0]-vl2[0], self[1]-vl2[1], self[2]-vl2[2]])
    def __mul__(self, value):
        # vec1 * value
        return vector([self[0]*value, self[1]*value, self[2]*value])
    def __div__(self, value):
        # vec1 / value
        return vector([self[0]/value, self[1]/value, self[2]/value])
    def __getitem__(self, index):
        # vec1[index]
        try:
            return self.vl[int(index)]
        except (ValueError, IndexError):
            try:
                return self.vl[{'x':0,'y':1,'z':2}[str(index)]]
            except KeyError:
                raise IndexError("Index must be 0, 1, 2, 'x', 'y' or 'z'")
    def __setitem__(self, index, value):
        # vec1[index] = value
        try:
            self.vl[int(index)] = float(value)
        except (ValueError, IndexError):
            try:
                self.vl[{'x':0,'y':1,'z':2}[str(index)]] = float(value)
            except KeyError:
                raise IndexError("Index must be 0, 1, 2, 'x', 'y' or 'z'")
    def __eq__(self, vec2):
        # vec1 == vec2
        return self.vl == vec2.vl
    def __ne__(self, vec2):
        # vec1 != vec2
        return not self.__eq__(vec2)
    def __neg__(self):
        # -vec1
        return vector([x.__neg__() for x in self.vl])
    
    def copy(self):
        # return identical vector
        return vector(self.vl)
    
    def ip(self, vec2):
        return ip(self, vec2)
    def cp(self, vec2):
        return cp(self, vec2)
    def angle(self, vec2):
        return angle(self, vec2)
    def angles(self, vec2):
        return angles(self, vec2)
    def length(self):
        return length(self)
    def setlength(self, newlength):
        return setlength(self, newlength)
    def normalize(self):
        return normalize(self)

def ip(vec1, vec2):
    # inner production
    return vec1[0]*vec2[0]+vec1[1]*vec2[1]+vec1[2]*vec2[2]
def cp(vec1, vec2):
    # cross production
    x = vec1[1]*vec2[2]-vec1[2]*vec2[1]
    y = vec1[2]*vec2[0]-vec1[0]*vec2[2]
    z = vec1[0]*vec2[1]-vec1[1]*vec2[0]
    return vector([x,y,z])
def angle(vec1, vec2):
    # angle between the vectors
    try:
        return math.acos(ip(vec1, vec2)/(length(vec1)*length(vec2)))
    except ZeroDivisionError:
        raise ZeroDivisionError("Cannot calculate angle for zero vector")
def angles(vec1, vec2):
    # list of angles in three planes
    noxip = vec1[1]*vec2[1]+vec1[2]*vec2[2]
    noxl1 = math.sqrt(math.pow(vec1[1],2)+math.pow(vec1[2],2))
    noxl2 = math.sqrt(math.pow(vec2[1],2)+math.pow(vec2[2],2))
    noyip = vec1[0]*vec2[0]+vec1[2]*vec2[2]
    noyl1 = math.sqrt(math.pow(vec1[0],2)+math.pow(vec1[2],2))
    noyl2 = math.sqrt(math.pow(vec2[0],2)+math.pow(vec2[2],2))
    nozip = vec1[1]*vec2[1]+vec1[0]*vec2[0]
    nozl1 = math.sqrt(math.pow(vec1[1],2)+math.pow(vec1[0],2))
    nozl2 = math.sqrt(math.pow(vec2[1],2)+math.pow(vec2[0],2))
    try:
        nox = math.acos(noxip/(noxl1*noxl2))
    except ZeroDivisionError:
        nox = None
    try:
        noy = math.acos(noyip/(noyl1*noyl2))
    except ZeroDivisionError:
        noy = None
    try:
        noz = math.acos(nozip/(nozl1*nozl2))
    except ZeroDivisionError:
        noz = None
    return [nox, noy, noz]
def length(vec1):
    # vector length
    return math.sqrt(math.pow(vec1[0],2)+math.pow(vec1[1],2)+math.pow(vec1[2],2))
def setlength(vec1, newlength):
    try:
        mod = newlength/length(vec1)
    except ZeroDivisionError:
        raise ZeroDivisionError("Cannot set length of zero vector")
    return vec1 * mod
def normalize(vec1):
    # set length to 1
    return setlength(vec1, 1)

# some helper functions
def distance(coord1, coord2):
    return length(vector(coord1)-vector(coord2))
def viewangles(coord1, coord2, roll=0.0):
    # note: vector(vector) works also, so you can pass vectors also ;)
    v1 = vector(coord1)
    v2 = vector(coord2)
    height = v2[2]-v1[2]
    xl = v2[0]-v1[0]
    yl = v2[1]-v1[1]
    xylen = math.hypot(xl,yl)
    try:
        pitch = math.degrees(math.atan(height/xylen))
    except ZeroDivisionError:
        pitch = cmp(v2[2],v1[2])*90.0
    try:
        yaw = math.degrees(math.atan(yl/xl))
    except ZeroDivisionError:
        yaw = cmp(v2[1],v1[1])*90.0
    return [pitch, yaw, float(roll)]
def viewvector(va):
    # creates a vector from [pitch, yaw, roll] list
    vangles = vector(va)
    x = math.cos(math.radians(vangles[1]))
    y = math.sin(math.radians(vangles[1]))
    z = math.sin(math.radians(vangles[0]))
    return vector([x,y,z])
def isbetweenRect(what, corner1, corner2):
    minvector = vector((
        min((corner1[0],corner2[0])),
        min((corner1[1],corner2[1])),
        min((corner1[2],corner2[2]))
        ))
    maxvector = vector((
        max((corner1[0],corner2[0])),
        max((corner1[1],corner2[1])),
        max((corner1[2],corner2[2]))
        ))
    for i in (0,1,2):
        if minvector[i] > what[i] or what[i] > maxvector[i]:
            return False
    return True
def isbetweenVect(what, corner1, corner2):
    v = vector(what)
    c1 = vector(corner1)
    c2 = vector(corner2)
    if (v==c1 or v==c2):
        return True
    if c1==c2:
        return False
    return ((v[0]-c1[0])/(v[1]-c1[1])==(c2[0]-c1[0])/(c2[1]-c1[1]) and
            (v[0]-c1[0])/(v[2]-c1[2])==(c2[0]-c1[0])/(c2[2]-c1[2]) and
            (v[2]-c1[2])/(v[1]-c1[1])==(c2[2]-c1[2])/(c2[1]-c1[1]))
